# No Comment --- Comment any resource on the web!
# Copyright © 2023 Bioneland
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

from pathlib import Path

from flask import current_app

from no_comment.domain.commenting.services import Calendar
from no_comment.infrastructure.file_system.repositories import Comments


def comments() -> Comments:
    return Comments(Path(current_app.config.get("DATA_DIR", "")))


def calendar() -> Calendar:
    # TODO: read timezone info from service and/or server configuration.
    return Calendar()
